from importlib.metadata import version

# Version string for the SOSAA simulation
sosaa_version = version(__package__)
# Pretty-printed version string for the SOSAA GUI
sosaa_version_pretty = f"SOSAA Model {sosaa_version} GUI"
